// @ myth - client - global.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "client/global.h"
#include "common/format.h"
#include "common/config.h"
#include "common/parse.h"

// @@ MATCH

// @@@ class MATCH

class MATCH
{
    const char* env;
    void SkipWhiteSpace();
public:
    bool operator()(const char* name);
    void next();
    STRING string();
    long value();
    bool done();
    MATCH(const char* string);
};

// @@@ void MATCH::SkipWhiteSpace()

void MATCH::SkipWhiteSpace()
{
    while(*env == ' ' || *env == '\t')
	env++;
}

// @@@ bool MATCH::operator()(const char* name)

bool MATCH::operator()(const char* name)
{ return strncmp(env, name, strlen(name)) == EQUAL; }

// @@@ bool MATCH::done()

bool MATCH::done()
{ return env == NULL || *env == '\0'; }

// @@@ void MATCH::next()

void MATCH::next()
{
    char c;

    while((c = *env) != '\0')
    {
	env++;
	if(c == ':')
	{
	    SkipWhiteSpace();
	    break;
	}
    }
}

// @@@ STRING MATCH::string()

STRING MATCH::string()
{
    // Skip the name of the option
    char c;
    do
    {
	c = *env;
	if(c == '\0' || c == ':')
	    return NullString;
	env++;
    }
    while(c != '=');

    SkipWhiteSpace();

    // Return the right side value
    char* end = strchr(env, ':');

    if(end == NULL)
	return STRING(env);
    else
    {
	char buffer[MessageSize];

	for(char* s = buffer; env < end; s++, env++)
	    *s = *env;

	*s = '\0';

	return STRING(buffer);
    }
}

// @@@ long MATCH::value()

long MATCH::value()
{ return atol(string().string()); }

// @@@ MATCH::MATCH(const char* string)

MATCH::MATCH(const char* string)
{
    if((env = string) != NULL)
	SkipWhiteSpace();
}

// @@ GLOBAL

// @@@ void GLOBAL::usage()

void GLOBAL::usage()
{
    (void) fprintf(stderr, "Usage: myth -n name -h host -s -S -b width");
    exit(1);
}

// @@@ err GLOBAL::parse(const char* string)

err GLOBAL::parse(const char* string)
{
    for(MATCH match(string); !match.done(); match.next())
    {
	if(match("name"))
	    MyName = match.string();
	else if(match("host"))
	    MyHost = match.string();
	else if(match("sticky"))
	    sticky_menues = TRUE;
	else if(match("nosticky"))
	    sticky_menues = FALSE;
	else if(match("border"))
	    border_width = match.value();
	else
	    return ERROR;
    }
    return SUCCES;
}    

// @@@ STRING GLOBAL::LibraryFile(const char* name)

STRING GLOBAL::LibraryFile(const char* name)
{ return format("%s/%s/%s", HomeDirectory, LibraryDirectory, name); }
    

// @@@ STRING GLOBAL::HostFile()

STRING GLOBAL::HostFile()
{ return LibraryFile("host"); }

// @@@ STRING GLOBAL::ServerFile()

STRING GLOBAL::ServerFile()
{ return LibraryFile("mythd"); }

// @@@ STRING GLOBAL::ErrorFile()

STRING GLOBAL::ErrorFile()
{
    return format("%s/%s/ERR.%d",
		  HomeDirectory, ErrorDirectory, geteuid()); 
}

// @@@ STRING GLOBAL::name()

STRING GLOBAL::name()
{ return MyName; }

// @@@ STRING GLOBAL::host()

STRING GLOBAL::host()
{ return MyHost; }

// @@@ int GLOBAL::border()

int GLOBAL::border()
{ return border_width; }

// @@@ bool GLOBAL::sticky()

bool GLOBAL::sticky()
{ return sticky_menues; }

// @@@ void GLOBAL::load(int argc, char** argv)

void GLOBAL::load(int argc, char** argv)
{
    // Parse arguments

    for(;;)
    {
	argc--;			// Skip the name of the game
	argv++;

	if(argc < 1)		// Last argument?
	    break;

	if((*argv)[0] == '-')
	{
	    switch((*argv)[1])
	    {
	    case 'n':
		if(argc < 2)
		    usage();
		argc--;
		argv++;
		MyName = STRING(*argv);
		break;

	    case 'h':
		if(argc < 2)
		    usage();
		argc--;
		argv++;
		MyHost = STRING(*argv);
		break;

	    case 's':
		sticky_menues = TRUE;
		break;

	    case 'S':
		sticky_menues = FALSE;
		break;

	    case 'b':
		if(argc < 2)
		    usage();
		argc--;
		argv++;
		border_width = atoi(*argv);
		break;
	    default:
		usage();
	    }
	}
	else
	    usage();
    }

    // Make sure we got the name

    if(MyName == NullString)
    {
	char* name;
	
	if((name = getenv("USER")) != NULL ||
	   (name = getenv("LOGNAME")) != NULL ||
	   (name = getlogin()) != NULL)
	{
	    MyName = STRING(name);
	}
	else
	    MyName = STRING("alien");
    }

    // Make sure we found a host

    if(MyHost == NullString)
    {
	PARSE LogFile(HostFile());
	
	if(LogFile.good())
	    MyHost = STRING(LogFile.GetWord());
	else
	{
	    (void) fprintf(stderr, "Starting local daemon");
	    (void) fflush(stderr);
	    // Actually, mythd should fork, and exit when ready.
	    (void) system(format("%s &", ServerFile().string()).string());
	    for(int i = 0; i < 10; i++)
	    {
		(void) fprintf(stderr, ".");
		(void) fflush(stderr);
		sleep(1);
	    }
	    (void) fprintf(stderr, "done\n");
	    (void) fflush(stderr);
	    char buffer[MessageSize];
	    (void) gethostname(buffer, MessageSize);
	    MyHost = STRING(buffer);
	}
    }
}

// @@@ GLOBAL::GLOBAL()

GLOBAL::GLOBAL()
{
    // Default values
    sticky_menues = TRUE;
    border_width = 3;

    // Parse the MYTH environment variable
    if(parse(getenv("MYTH")) == ERROR) // Handle NULL ok...
	(void) fprintf(stderr, "Garbage in MYTH environment.\n");
}

// @@ GLOBAL global

GLOBAL global;

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
