// @ myth - client - myth.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// This file contain the main loop of the client part of myth.
//
// It relies on three objects:
//    SERVER, which handle the connection to the server process, 
//    INPUT, which read the players input from the keyboard and
//    OUTPUT, which handle the output to the screen.
//
// The screen is divided into five pieces, which each have a handler
// as a part of the OUTPUT object.
//    PROMPT,
//    STATUS,
//    NEWS,
//    LEVEL and
//    INVENTORY.
//
// Part of the screen may be temporarily covered by using the COVER
// object.
//
// An outline of the program is:
//    
// While playing
//    Wait for an request from the server
//    Until the request is answered
//       Read and process input from keyboard and server
//    Answer request
//
// The object READY is used to examine if there is data ready from
// either the server or the keyboard.

// @@ include

#include "client/myth.h"
#include "common/error.h"
#include "common/wait.h"
#include "client/server.h"
#include "client/input.h"
#include "client/output.h"
#include "common/format.h"
#include "client/global.h"

#include <signal.h>

// @@ module TTY

// @@@ TTY* GlobalTTY

TTY* GlobalTTY = NULL;

// @@@ void TTY_OutPut(char c)

inline void TTY_OutPut(char c)
{ GlobalTTY->put(c); }

// @@@ void CleanUpTTY()

static void CleanUpTTY()
{
    if(GlobalTTY != NULL)
	delete GlobalTTY;

    exit(0);
}

// @@@ void SelectTTY(TTY* tty)
//

static void SelectTTY(TTY* tty)
{
    if(tty == NULL)
	CleanUpTTY();
    else
    {
	(void) signal(SIGQUIT, (SIG_PF) CleanUpTTY);
	(void) signal(SIGINT, SIG_IGN);
	GlobalTTY = tty;
    }
}

// @@ class READY

// @@@ enum input_type

enum input_type
{ READY_SERVER, READY_KEYBOARD, READY_ERROR = -1 };

// @@@ class READY

class READY : WAIT
{
    const BUFFER* server;
    const BUFFER* keyboard;
public:
    input_type wait();
    READY(BUFFER* server, BUFFER* keyboard);
};

// @@@ input_type READY::wait()

input_type READY::wait()
{
    check();

    if(server->IsReady)
	return READY_SERVER;
    if(keyboard->IsReady)
	return READY_KEYBOARD;

    bug.error("READY::wait", "Not Reached");
    return READY_ERROR;
}

// @@@ READY::READY(BUFFER* server, BUFFER* keyboard)

READY::READY(BUFFER* server,  BUFFER* keyboard)
{
    READY::server = server;
    READY::keyboard = keyboard;
    add(server);
    add(keyboard);
}

// @@ void play(INPUT* input, OUTPUT* output)

void play(CHANNEL* server, INPUT* input, OUTPUT* output)
{
    READY ready(server, GlobalTTY);
    
    for(;;)
    {
	while(!input->ready())
	{
	    CheckServer(server, input, output);
	    if(output->status.playing())
	    {
		input->update(output);
		GlobalTTY->flush();
	    }
	    else
		return;

	    FormatCleanUp();	// Clean up temporaries for formating
	}
	do
	{
	    while(input->active())
	    {
		input->update(output);
		GlobalTTY->flush();

		switch(ready.wait())
		{
		case READY_SERVER:
		    CheckServer(server, input, output);
		    if(!output->status.playing())
			return;
		    break;
		case READY_KEYBOARD:
		    input->read(GlobalTTY->get(), output->cover.active());
		    break;
		case READY_ERROR:
		default:
		    bug.error("::play", "Unknown value for ready.wait()");
		}
		FormatCleanUp(); // Clean up temporaries for formating
	    }
	}
	while(input->check(output));

	input->answer(server);
    }
}

// @@ int main(int argc, char** argv)

int main(int argc, char** argv)
{
    global.load(argc, argv);
    bug.open(global.ErrorFile());
    CHANNEL server(global.host().string(), StepDaemon);

    if(server.error())
    {
	bug.fatal("::main",
		  format("Could not connect to %s", global.host().string()));
    }
    SelectTTY(new TTY("/dev/tty"));
    set_output_char(TTY_OutPut);
    SCREEN screen;
    SelectScreen(&screen);
    INPUT input;
    OUTPUT output;

    play(&server, &input, &output);

    // BUG: What if he change Q to something else?
    output.news.insert("Game Over --- Please press Q to quit.");

    input.EndGame();

    do
    {
	while(input.active())
	{
	    input.update(&output);
	    GlobalTTY->flush();
	    input.read(GlobalTTY->get(), output.cover.active());
	}
    }
    while(input.check(&output));
    
    screen.cover();
    screen.ClearScreen();
    GlobalTTY->flush();
    SelectTTY(NULL);
    return 0;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
