// @ myth - common - binary.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/binary.h"
#include "common/error.h"
#include "common/format.h"

// @@ BINARY

// @@@ void BINARY::create()

void BINARY::create()
{
    LongState = ShortState = CoorState = 0;
}    

// @@@ void BINARY::open(int fd)

void BINARY::open(int fd)
{ BUFFER::open(fd); }

// @@@ void BINARY::open(const char* name)

void BINARY::open(const char* name)
{ BUFFER::open(name); }

// @@@ void BINARY::PutChar(char c)

void BINARY::PutChar(char c)
{
// bug.warning("BINARY::PutChar", form("Put %d (== %c)", c, c));
    put(&c, 1);
}

// @@@ void BINARY::PutShort(short i)
//
// Send short int coded in 15 bit plus sign bit

void BINARY::PutShort(short i)
{
    bool negative = (i < 0);

    if(negative)
	i = -i;

    PutChar(i & 0xff);
    PutChar(((i >> 8) & 0x7f) | (negative << 7));
}

// @@@ void BINARY::PutLong(long l)
//
// Send long int coded in 31 bit plus sign bit

void BINARY::PutLong(long l)
{
    bool negative = (l < 0L);

    if(negative)
	l = -l;

    PutChar(l & 0xff);
    PutChar((l >>= 8) & 0xff);
    PutChar((l >>= 8) & 0xff);
    PutChar(((l >> 8) & 0x7f) | (negative << 7));
}    

// @@@ void BINARY::PutString(const char* s)
//
// Send a null terminated string to the buffer

void BINARY::PutString(const char* s)
{
    while(*s)
    {
	PutChar(*s);
	s++;
    }
    PutChar(0);
}

// @@@ void BINARY::PutCoor(coor pos)

void BINARY::PutCoor(coor pos)
{
    PutShort(pos.x);
    PutShort(pos.y);
}

// @@@ bool BINARY::IsShortReady()

bool BINARY::IsShortReady()
{
    switch(ShortState)
    {
    case 0:
	ShortChar[ShortState] = GetChar();
	ShortState++;
	return FALSE;
    case 1:
	ShortChar[ShortState] = GetChar();
	ShortState++;
	return TRUE;
    case 2:
	return TRUE;
    default:
	bug.error("BINARY::IsShortReady", format("state %d", ShortState));
	return TRUE;
    }
}

// @@@ bool BINARY::IsLongReady()

bool BINARY::IsLongReady()
{
    switch(LongState)
    {
    case 0:
    case 1:
    case 2:
	LongChar[LongState] = GetChar();
	LongState++;
	return FALSE;
    case 3:
	LongChar[LongState] = GetChar();
	LongState++;
	return TRUE;
    case 4:
	return TRUE;
    default:
	bug.error("BINARY::IsLongReady", format("state %d", LongState));
	return TRUE;
    }
}

// @@@ bool BINARY::IsCoorReady()

bool BINARY::IsCoorReady()
{
    switch(CoorState)
    {
    case 0:
	if(IsShortReady())
	{
	    CoorShort[CoorState] = GetShort();
	    CoorState++;
	}
	return FALSE;
    case 1:
	if(IsShortReady())
	{
	    CoorShort[CoorState] = GetShort();
	    CoorState++;
	    return TRUE;
	}
	else
	    return FALSE;
    case 2:
	return TRUE;
    default:
	bug.error("BINARY::IsCoorReady", format("state %d", CoorState));
	return TRUE;
    }
}

// @@@ char BINARY::GetChar()
//
// Read char from buffer

char BINARY::GetChar()
{
    char c;
    get(&c, 1);
// bug.warning("BINARY::GetChar", form("Got %d (== %c)", c, c));
    return c;
}

// @@@ short BINARY::GetShort()
//
// Read a 15 bit plus sign bit integer

short BINARY::GetShort()
{
    while(!IsShortReady())
	/* Do Nothing! */;

    // Clear the state (Make ready for next short)
    ShortState = 0;
    // Concatenate the unsigned short
    short i = ((ShortChar[1] & 0x7f) << 8) | (ShortChar[0] & 0xff);
    // Extract the sign bit
    bool negative = ShortChar[1] & 0x80;
    // Return the answer
    return negative ? -i : i;
}

// @@@ long BINARY::GetLong()
//
// Read a 31 bit plus sign bit integer

long BINARY::GetLong()
{
    while(!IsLongReady())
	/* Do Nothing! */;

    // Prepare for next long integer
    LongState = 0;
    // Concatenate the unsigned long
    long l = (LongChar[0] & 0xff) | ((LongChar[1] & 0xff) << 8) |
	((LongChar[2] & 0xff) << 16) | ((LongChar[3] & 0x7f) << 24);
    // Extract the sign from Most Significant Byte
    bool negative = LongChar[3] & 0x80;
    // Return the signed answer
    return negative ? -l : l;
}

#if 0

// @@@ char* BINARY::GetString(char* buffer, int size)
//
// Read a nul terminated string from the buffer.
//
// The string is read into a buffer, which must be at least size long.
// A pointer to the buffer is returned.
//
// If the buffer is to small, it will be terminated with a null, and
// the remaining part of the string will be skiped.

char* BINARY:: GetString(char* buffer, int size)
{
    char* s = buffer;
    char c = GetChar();
    
    while(--size && c)
    {
	*s = c;
	s++;
	c = GetChar();
    }
    *s = 0;

    while(c)			// skip remaining part of string
	c = GetChar();

    return buffer;
}

#endif

// @@@ STRING BINARY::GetString()
//
// Read a NUL terminated string from the buffer.
// The string returned is at most MessageSize long, inclusive the NUL.
// Any overflow will be skiped.

STRING BINARY:: GetString()
{
    static char buffer[MessageSize];
    char* s = buffer;
    int size = MessageSize - 1;

    char c = GetChar();
    
    while(--size && c)
    {
	*s = c;
	s++;
	c = GetChar();
    }
    *s = 0;

    while(c)			// skip remaining part of string
	c = GetChar();

    return STRING(buffer);
}

// @@@ coor BINARY::GetCoor()

coor BINARY::GetCoor()
{
    while(!IsCoorReady())
	/* Do Nothing! */;

    // Prepare for next coor
    CoorState = 0;
    // Return the new coor
    return coor(CoorShort[0], CoorShort[1]);
}

// @@@ BINARY::BINARY(buffer_mode mode, const char* name)

BINARY::BINARY(buffer_mode mode, const char* name) : (mode, name)
{ create(); }

// @@@ BINARY::BINARY(buffer_mode mode, int fd = ERROR)

BINARY::BINARY(buffer_mode mode, int fd = ERROR) : (mode, fd)
{ create(); }

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
