// @ myth - common - command.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef COMMAND_HEADER
#define COMMAND_HEADER

// @@ include

#include "common/common.h"
#include "common/symbol.h"

// @@ enum from_client

// Maximum number of arguments to a client command
static const int MaxCommandLength = 2;

enum from_client
{
    CLIENT_IDLE,		// ()		- ignore this
    // The following is regul{r commands, send by client on request by player
    CLIENT_MOVE,		// (direction)	- move or examine
    CLIENT_RUN,			// (direction)	- run, fight, dig or pick up
    CLIENT_DROP,		// (object)
    CLIENT_GET,			// ()
    CLIENT_APPLY,		// (object)     - wear, read, quaff, eat etc...
    CLIENT_THROW,		// (object, location) - throw or shoot
    CLIENT_SLEEP,		// (confirmation)- save
    CLIENT_COMMAND,		// (string)	- extended command
    // The following commands is only used inside the client
    CLIENT_RECENTER,		// (location)	- center dungeon window
    CLIENT_INVENTORY,		// (symbol)	- show inventory
    CLIENT_SHOW_STATUS,		// ()		- move top of display window
    CLIENT_READ_NEWS,		// ()		- show new messages
    CLIENT_NEWS_UP,		// ()		- scroll news window up
    CLIENT_NEWS_DOWN,		// ()		- scroll news window down
    // The following commands is only used inside the server
    // Misc.
    CLIENT_END,			// ()		- number of commands
    CLIENT_ERROR = ERROR	// ()		- error condition
};

// @@ enum from_server

enum from_server
{
    SERVER_IDLE,		// ()		- ignore this
    // To the message handler
    SERVER_NEWS,		// (string)	- queued message to player
    // To the status handler
    SERVER_STATUS,		// (status, long) - set status
    // These is to the inventory handler
    SERVER_INVENTORY,           // (number, page) - set inventory[number]
    // These is to the dungeon handler
    SERVER_MAP,			// ()		- clear dungeon map
    SERVER_HOME,		// (location)	- cursor position
    SERVER_PLOT,		// (location, char) - change dungeon map
    SERVER_PRINT,		// (location, string) - change dungeon map
    SERVER_EFFECT,		// (location, char) - special effect
    // These is to the prompt handler and the input handler
    SERVER_REQUEST,		// (prompt, request ...) - ask for info
    SERVER_PAGE,		// (page)       - bind page to previous req
    SERVER_READY,		// ()		- ready to accept answer
    // Misc.
    SERVER_END,			// ()		- end of server commands
    SERVER_ERROR = ERROR	// ()		- error condition
};

// @@ enum request_type

enum request_type
{
    REQUEST_IDLE,		// ()		- don't requere anything
    REQUEST_OBJECT,		// (char ...)	- object description
    REQUEST_POSITION,		// (short, short) - (coor)
    REQUEST_STRING,		// (string)
    REQUEST_DIRECTION,		// (char)	- (direction)
    REQUEST_SYMBOL,		// (char)
    REQUEST_CONFIRMATION,	// (char)	- (NoYesQuit)
    REQUEST_ARGUMENT,		// (long)
    REQUEST_LETTER,		// (char)
    REQUEST_NAME,		// (string)	- name of the player
    REQUEST_COMMAND,		// (char)  	- (from_client)
    REQUEST_END,		// Number of requests.
    REQUEST_ERROR = 127		// Error condition (must be a char).
};

// @@ enum request_answer

enum request_answer
{ ANSWER_ESCAPE, ANSWER, ANSWER_ARGUMENT };

// @@ enum prompt_type

enum prompt_type
{ PROMPT_IDLE,
  // The following prompt is used at the secondary prompt line
  PROMPT_SYMBOL, PROMPT_LETTER, PROMPT_POSITION, PROMPT_STRING,
  PROMPT_DIRECTION, PROMPT_CONFIRMATION, PROMPT_ARGUMENT, PROMPT_NAME,
  PROMPT_COMMAND,
  // The following prompt is used at the primary prompt line
  PROMPT_COPYRIGHT, PROMPT_WELCOME, PROMPT_PASSWORD,
  // The following is set by the server when asking questions
  PROMPT_IDENTIFY, PROMPT_TELEPORT, PROMPT_BLESS, PROMPT_MAGIC,
  PROMPT_ZAP, PROMPT_BREATHE, PROMPT_EAT,
  // These are set by the client when reading arguments to commands
  PROMPT_MOVE, PROMPT_RUN, PROMPT_DROP, PROMPT_GET, PROMPT_APPLY,
  PROMPT_THROW, PROMPT_SLEEP, PROMPT_EXTENDED, PROMPT_RECENTER,
  PROMPT_INVENTORY, 
  // misc
  PROMPT_END, PROMPT_ERROR = ERROR };

// @@ enum status_type

enum status_type
{
    // Client status
    STATUS_NEWS,
    // Game status
    STATUS_EXPERIENCE, STATUS_DUNGEON, STATUS_GAME,
    // Attributes
    STATUS_STRENGTH, STATUS_HEALTH,
    STATUS_FOOD,
    // Misc
    STATUS_END, STATUS_ERROR = ERROR,
    STATUS_CLIENT = STATUS_NEWS,
    STATUS_SERVER = STATUS_EXPERIENCE,
    STATUS_CLIENT_END = STATUS_SERVER,
    STATUS_SERVER_END = STATUS_END
};

// @@ typedef

typedef long status_value;

// @@ enum game_type
//
// Values >= GAME_OVER mean that the game is stopped.

enum game_type
{
    GAME_IDLE,			// He has not started yet
    GAME_WAIT,			// He may not fight right now...
    GAME_PLAY,			// He is playing
    GAME_OVER,			// The game is over
    GAME_SLEEP,			// He is sleeping
    GAME_END,
    GAME_ERROR = ERROR
};

// @@ enum inventory_type

enum inventory_type
{ INVENTORY_OWN, INVENTORY_OTHER, INVENTORY_END, INVENTORY_ERROR = ERROR };

// @@ extern

extern from_client CommandToType(const char* name);
extern status_type StatusToType(const char* name);
extern request_type RequestToType(const char* name);
extern prompt_type PromptToType(const char* name);

// @@ end

#endif COMMAND_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
